@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -66,7 +66,7 @@ extern "C"
  * Free a Authenication-Info element.
  * @param header The element to work on.
  */
-#define osip_proxy_authentication_info_free(header) osip_authentication_info_free(header)
+#define osip_proxy_authentication_info_free osip_authentication_info_free
 /**
  * Clone a Authenication-Info element.
  * @param header The element to work on.
