@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -59,7 +59,7 @@ extern "C"
  */
   struct __node
   {
-    void *next;			/**< next __node_t containing element */
+    void *next;                 /**< next __node_t containing element */
     void *element;              /**< element in Current node */
   };
 #endif
@@ -71,10 +71,11 @@ extern "C"
   typedef struct osip_list osip_list_t;
 
 /* added by bennewit@cs.tu-berlin.de */
-  typedef struct {
-    __node_t * actual;
-    __node_t ** prev;
-    osip_list_t * li;
+  typedef struct
+  {
+    __node_t *actual;
+    __node_t **prev;
+    osip_list_t *li;
     int pos;
   } osip_list_iterator_t;
 
@@ -110,6 +111,14 @@ extern "C"
  * @param free_func The method that is able to release one element of the list.
  */
   void osip_list_special_free (osip_list_t * li, void *(*free_func) (void *));
+/**
+ * Clone a list of element.
+ * Each element will be cloned with the method given as the second parameter.
+ * @param src The element to work on.
+ * @param dst The element to work on.
+ * @param clone_func The method that is able to release one element of the list.
+ */
+  int osip_list_clone (const osip_list_t * src, osip_list_t * dst, int *(*clone_func) (void *, void *));
 /**
  * Free a list of element where elements are pointer to 'char'.
  * @param li The element to work on.
@@ -147,11 +156,11 @@ extern "C"
   int osip_list_remove (osip_list_t * li, int pos);
 
 /* added by bennewit@cs.tu-berlin.de */
-  void * osip_list_get_first( osip_list_t * li, osip_list_iterator_t * it );
+  void *osip_list_get_first (osip_list_t * li, osip_list_iterator_t * it);
 /* added by bennewit@cs.tu-berlin.de */
-  void * osip_list_get_next( osip_list_iterator_t * it );
+  void *osip_list_get_next (osip_list_iterator_t * it);
 /* added by bennewit@cs.tu-berlin.de */
-  void * osip_list_iterator_remove( osip_list_iterator_t * it );
+  void *osip_list_iterator_remove (osip_list_iterator_t * it);
 
 #ifdef __cplusplus
 }
