@@ -31,7 +31,7 @@ typedef struct osip_statemachine osip_statemachine_t;
 
 struct osip_statemachine
 {
-  osip_list_t *transitions;
+  struct _transition_t *transitions;
 };
 
 /**
@@ -48,6 +48,7 @@ type_t evt_set_type_incoming_sipmessage (osip_message_t * sip);
 type_t evt_set_type_outgoing_sipmessage (osip_message_t * sip);
 
 
+#ifndef MINISIZE
 void __ict_load_fsm (void);
 void __ist_load_fsm (void);
 void __nict_load_fsm (void);
@@ -64,14 +65,18 @@ osip_statemachine_t *__ist_get_fsm (void);
 
 osip_statemachine_t *__nict_get_fsm (void);
 osip_statemachine_t *__nist_get_fsm (void);
+#endif
 
-typedef struct _transition_t
+typedef struct _transition_t transition_t;
+
+struct _transition_t
 {
   state_t state;
   type_t type;
   void (*method) (void *, void *);
-}
-transition_t;
+  struct _transition_t *next;
+  struct _transition_t *parent;
+};
 
 int fsm_callmethod (type_t type, state_t state,
                     osip_statemachine_t * statemachine,
@@ -101,8 +106,6 @@ void ict_retransmit_ack (osip_transaction_t * ict, osip_event_t * evt);
 /* FSM  ---- > IST      */
 /************************/
 
-osip_message_t *ist_create_resp_100 (osip_transaction_t * ist,
-                                     osip_message_t * request);
 void ist_rcv_invite (osip_transaction_t * ist, osip_event_t * evt);
 void osip_ist_timeout_g_event (osip_transaction_t * ist, osip_event_t * evt);
 void osip_ist_timeout_h_event (osip_transaction_t * ist, osip_event_t * evt);
@@ -173,6 +176,15 @@ __osip_transaction_matching_request_osip_to_xist_17_2_3 (osip_transaction_t *
                                                          tr,
                                                          osip_message_t * request);
 
+osip_event_t *
+__osip_transaction_need_timer_x_event (void *xixt, struct timeval *timer,
+				       int cond_state,
+				       int transactionid,
+				       int TIMER_VAL);
+
+int
+__osip_transaction_snd_xxx (osip_transaction_t * ist, osip_message_t * msg);
+
 #endif
 
 #endif
