@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -92,11 +92,20 @@ __osip_nict_init (osip_nict_t ** nict, osip_t * osip, osip_message_t * request)
   } else
     {
       int port = 5060;
-
+      /* search for maddr parameter */
+      osip_uri_param_t *maddr_param = NULL;
+      
+      port = 5060;
       if (request->req_uri->port != NULL)
         port = osip_atoi (request->req_uri->port);
-      osip_nict_set_destination ((*nict),
-                                 osip_strdup (request->req_uri->host), port);
+      
+      osip_uri_uparam_get_byname (request->req_uri, "maddr", &maddr_param);
+      if (maddr_param!=NULL && maddr_param->gvalue!=NULL)
+	osip_nict_set_destination ((*nict),
+				   osip_strdup (maddr_param->gvalue), port);
+      else
+	osip_nict_set_destination ((*nict),
+				   osip_strdup (request->req_uri->host), port);
     }
 
   (*nict)->timer_f_length = 64 * DEFAULT_T1;
@@ -142,58 +151,30 @@ osip_event_t *
 __osip_nict_need_timer_e_event (osip_nict_t * nict, state_t state,
                                 int transactionid)
 {
-  struct timeval now;
-
-  osip_gettimeofday (&now, NULL);
-
-  if (nict == NULL)
-    return NULL;
-  if (state == NICT_PROCEEDING || state == NICT_TRYING)
-    {
-      if (nict->timer_e_start.tv_sec == -1)
-        return NULL;
-      if (osip_timercmp (&now, &nict->timer_e_start, >))
-        return __osip_event_new (TIMEOUT_E, transactionid);
-    }
-  return NULL;
+  return __osip_transaction_need_timer_x_event(nict, &nict->timer_e_start,
+					       state == NICT_PROCEEDING
+					       || state == NICT_TRYING,
+					       transactionid,
+					       TIMEOUT_E);
 }
 
 osip_event_t *
 __osip_nict_need_timer_f_event (osip_nict_t * nict, state_t state,
                                 int transactionid)
 {
-  struct timeval now;
-
-  osip_gettimeofday (&now, NULL);
-
-  if (nict == NULL)
-    return NULL;
-  if (state == NICT_PROCEEDING || state == NICT_TRYING)
-    {
-      if (nict->timer_f_start.tv_sec == -1)
-        return NULL;
-      if (osip_timercmp (&now, &nict->timer_f_start, >))
-        return __osip_event_new (TIMEOUT_F, transactionid);
-    }
-  return NULL;
+  return __osip_transaction_need_timer_x_event(nict, &nict->timer_f_start,
+					       state == NICT_PROCEEDING
+					       || state == NICT_TRYING,
+					       transactionid,
+					       TIMEOUT_F);
 }
 
 osip_event_t *
 __osip_nict_need_timer_k_event (osip_nict_t * nict, state_t state,
                                 int transactionid)
 {
-  struct timeval now;
-
-  osip_gettimeofday (&now, NULL);
-
-  if (nict == NULL)
-    return NULL;
-  if (state == NICT_COMPLETED)
-    {
-      if (nict->timer_k_start.tv_sec == -1)
-        return NULL;
-      if (osip_timercmp (&now, &nict->timer_k_start, >))
-        return __osip_event_new (TIMEOUT_K, transactionid);
-    }
-  return NULL;
+  return __osip_transaction_need_timer_x_event(nict, &nict->timer_k_start,
+					       state == NICT_COMPLETED,
+					       transactionid,
+					       TIMEOUT_K);
 }
