@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,7 @@
 #include <osipparser2/osip_parser.h>
 #include "parser.h"
 
+#ifndef MINISIZE
 
 /* Accept-Encoding = token
    token possible values are gzip,compress,deflate,identity
@@ -48,7 +49,7 @@ osip_message_set_accept_encoding (osip_message_t * sip, const char *hvalue)
       return -1;
     }
   sip->message_property = 2;
-  osip_list_add (sip->accept_encodings, accept_encoding, -1);
+  osip_list_add (&sip->accept_encodings, accept_encoding, -1);
   return 0;
 }
 
@@ -59,10 +60,10 @@ osip_message_get_accept_encoding (const osip_message_t * sip, int pos,
   osip_accept_encoding_t *accept_encoding;
 
   *dest = NULL;
-  if (osip_list_size (sip->accept_encodings) <= pos)
+  if (osip_list_size (&sip->accept_encodings) <= pos)
     return -1;                  /* does not exist */
   accept_encoding =
-    (osip_accept_encoding_t *) osip_list_get (sip->accept_encodings, pos);
+    (osip_accept_encoding_t *) osip_list_get (&sip->accept_encodings, pos);
   *dest = accept_encoding;
   return pos;
 }
@@ -76,15 +77,7 @@ osip_accept_encoding_init (osip_accept_encoding_t ** accept_encoding)
     return -1;
   (*accept_encoding)->element = NULL;
 
-  (*accept_encoding)->gen_params =
-    (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  if ((*accept_encoding)->gen_params == NULL)
-    {
-      osip_free (*accept_encoding);
-      *accept_encoding = NULL;
-      return -1;
-    }
-  osip_list_init ((*accept_encoding)->gen_params);
+  osip_list_init (&(*accept_encoding)->gen_params);
 
   return 0;
 }
@@ -99,7 +92,7 @@ osip_accept_encoding_parse (osip_accept_encoding_t * accept_encoding,
 
   if (osip_accept_encoding_params != NULL)
     {
-      if (__osip_generic_param_parseall (accept_encoding->gen_params,
+      if (__osip_generic_param_parseall (&accept_encoding->gen_params,
                                          osip_accept_encoding_params) == -1)
         return -1;
   } else
@@ -143,10 +136,10 @@ osip_accept_encoding_to_str (const osip_accept_encoding_t * accept_encoding,
     size_t plen;
     osip_generic_param_t *u_param;
 
-    while (!osip_list_eol (accept_encoding->gen_params, pos))
+    while (!osip_list_eol (&accept_encoding->gen_params, pos))
       {
         u_param =
-          (osip_generic_param_t *) osip_list_get (accept_encoding->gen_params,
+          (osip_generic_param_t *) osip_list_get (&accept_encoding->gen_params,
                                                   pos);
         if (u_param->gvalue == NULL)
           plen = strlen (u_param->gname) + 2;
@@ -176,10 +169,9 @@ osip_accept_encoding_free (osip_accept_encoding_t * accept_encoding)
     return;
   osip_free (accept_encoding->element);
 
-  osip_generic_param_freelist (accept_encoding->gen_params);
+  osip_generic_param_freelist (&accept_encoding->gen_params);
 
   accept_encoding->element = NULL;
-  accept_encoding->gen_params = NULL;
   osip_free (accept_encoding);
 }
 
@@ -210,16 +202,16 @@ osip_accept_encoding_clone (const osip_accept_encoding_t * ctt,
     osip_generic_param_t *u_param;
     osip_generic_param_t *dest_param;
 
-    while (!osip_list_eol (ctt->gen_params, pos))
+    while (!osip_list_eol (&ctt->gen_params, pos))
       {
-        u_param = (osip_generic_param_t *) osip_list_get (ctt->gen_params, pos);
+        u_param = (osip_generic_param_t *) osip_list_get (&ctt->gen_params, pos);
         i = osip_generic_param_clone (u_param, &dest_param);
         if (i != 0)
           {
             osip_accept_encoding_free (ct);
             return -1;
           }
-        osip_list_add (ct->gen_params, dest_param, -1);
+        osip_list_add (&ct->gen_params, dest_param, -1);
         pos++;
       }
   }
@@ -239,3 +231,5 @@ osip_accept_encoding_set_element (osip_accept_encoding_t * ae, char *element)
 {
   ae->element = element;
 }
+
+#endif
