@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -48,7 +48,7 @@ osip_message_set_contact (osip_message_t * sip, const char *hvalue)
       return -1;
     }
   sip->message_property = 2;
-  osip_list_add (sip->contacts, contact, -1);
+  osip_list_add (&sip->contacts, contact, -1);
   return 0;                     /* ok */
 }
 
@@ -69,12 +69,15 @@ osip_contact_parse (osip_contact_t * contact, const char *hvalue)
   return osip_from_parse ((osip_from_t *) contact, hvalue);
 }
 
+#ifndef MINISIZE
 int
 osip_contact_init (osip_contact_t ** contact)
 {
   return osip_from_init ((osip_from_t **) contact);
 }
+#endif
 
+#ifndef MINISIZE
 /* returns the pos of contact header.                      */
 /* INPUT : int pos | pos of contact header.                */
 /* INPUT : osip_message_t *sip | sip message.                       */
@@ -87,11 +90,12 @@ osip_message_get_contact (const osip_message_t * sip, int pos,
   *dest = NULL;
   if (sip == NULL)
     return -1;
-  if (osip_list_size (sip->contacts) <= pos)
+  if (osip_list_size (&sip->contacts) <= pos)
     return -1;                  /* does not exist */
-  *dest = (osip_contact_t *) osip_list_get (sip->contacts, pos);
+  *dest = (osip_contact_t *) osip_list_get (&sip->contacts, pos);
   return pos;
 }
+#endif
 
 /* returns the contact header as a string.*/
 /* INPUT : osip_contact_t *contact | contact.  */
@@ -112,6 +116,7 @@ osip_contact_to_str (const osip_contact_t * contact, char **dest)
   return osip_from_to_str ((osip_from_t *) contact, dest);
 }
 
+#ifndef MINISIZE
 /* deallocates a osip_contact_t structure.  */
 /* INPUT : osip_contact_t *| contact. */
 void
@@ -125,3 +130,5 @@ osip_contact_clone (const osip_contact_t * contact, osip_contact_t ** dest)
 {
   return osip_from_clone ((osip_from_t *) contact, (osip_from_t **) dest);
 }
+#endif
+
