@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,11 +24,13 @@
 #include <osipparser2/osip_message.h>
 #include <osipparser2/osip_parser.h>
 
+#ifndef MINISIZE
 int
 osip_to_init (osip_to_t ** to)
 {
   return osip_from_init ((osip_from_t **) to);
 }
+#endif
 
 /* adds the to header to message.              */
 /* INPUT : const char *hvalue | value of header.    */
@@ -58,6 +60,7 @@ osip_message_set_to (osip_message_t * sip, const char *hvalue)
   return 0;
 }
 
+#ifndef MINISIZE
 /* returns the to header.            */
 /* INPUT : osip_message_t *sip | sip message.   */
 /* returns null on error. */
@@ -66,6 +69,9 @@ osip_message_get_to (const osip_message_t * sip)
 {
   return sip->to;
 }
+#endif
+
+#ifndef MINISIZE
 
 int
 osip_to_parse (osip_to_t * to, const char *hvalue)
@@ -101,3 +107,5 @@ osip_to_tag_match (osip_to_t * to1, osip_to_t * to2)
 {
   return osip_from_tag_match ((osip_from_t *) to1, (osip_from_t *) to2);
 }
+
+#endif
